ERROR_UNKNOWN_CASE_TYPE = "Unknown case type: {type_}, allowed types: {allowed}"
ERROR_WRONG_TYPE_CASE_TYPE = "Expected CaseType or str, got {type_}"
ERROR_INVALID_JSON = "Invalid JSON provided to transform_from_json: {msg}"
ERROR_NOT_DICT = "Expected dict to transform, got {type_}"
